"""Error classes for authentise_services"""


class ResourceError(Exception):
    """arbitrary error whenever a call to a authentise resource doesnt go according to plan"""
    pass


class ResourceStillProcessing(Exception):
    """most authentise resources have a status property to tell the user what state its in
        Whenever the resource isnt ready in some way or another, throw one of these"""
    pass



"""Config class module"""
import configparser  # pylint: disable=import-error
import os


class Config(object):
    """This class is for the modules that need authentise session and host info"""

    def __init__(self, path=None):
        self.path = path or self.find_config_file(["/srv/authentise_services/config.ini",
                                                   os.path.expanduser(
                                                       "~/.authentise_services/config.ini"),
        ])

        config = self.parse_config(self.path)

        self.host = config.get("host", "authentise.com")
        self.username = config.get("username", None)
        self.password = config.get("password", None)

    def __dir__(self):
        return {"host": self.host, "username": self.username, "password": self.password}

    @staticmethod
    def parse_config(path):
        """parse either the config file we found, or use some canned defaults"""
        config = configparser.ConfigParser()

        if path:  # if user has config with user creds in it, this will grab it
            config.read(path)

        try:
            return {k: v for k, v in config["default"].items()}
        except KeyError:
            return {}

    @staticmethod
    def find_config_file(possible_paths):
        """given a list of different paths for different systems, return the first one we find"""
        for path in possible_paths:
            if os.path.isfile(path):
                return path



"""Session class module"""
import requests

from authentise_services import errors
from authentise_services.config import Config


class Session(object):  # pylint: disable=too-few-public-methods
    """This class is for creating and holding onto user sessions for the model warehouse service"""

    def __init__(self, username=None, password=None):
        self.session = self.__create_session(username, password)

    def __str__(self):
        return self.session

    @staticmethod
    def __create_session(username=None, password=None):
        """grabs the configuration, and makes the call to Authentise to create the session"""
        config = Config()

        if not username or not password:
            username = config.username
            password = config.password

        payload = {
            "username": username,
            "password": password,
        }

        session_resp = requests.post("https://users.{}/sessions/".format(config.host), json=payload)
        if session_resp.status_code == 403:
            raise errors.ResourceError("bad user credentials")

        return session_resp.cookies["session"]

    @classmethod
    def create_user(cls, username, password, name, email):
        """utility class method to create a user"""
        config = Config()
        payload = {"username": username,
                   "email": email,
                   "name": name,
                   "password": password, }
        user_creation_resp = requests.post("https://users.{}/users/".format(config.host),
                                           json=payload)
        if user_creation_resp.status_code != 201:
            raise errors.ResourceError("couldnt create user")



import re
import subprocess

from setuptools import setup


def _get_git_description():
    try:
        return subprocess.check_output(["git", "describe"]).decode("utf-8").strip()
    except subprocess.CalledProcessError:
        return None


def _create_version_from_description(git_description):
    match = re.match(r'(?P<tag>[\d\.]+)-(?P<offset>[\d]+)-(?P<sha>\w{8})', git_description)
    if not match:
        version = git_description
    else:
        version = "{tag}.post{offset}".format(**match.groupdict())
    return version


def get_version():
    description = _get_git_description()

    return _create_version_from_description(description)


def main():
    setup(
        name="authentise_services",
        url="https://github.com/DoWhileGeek/authentise-services",
        description="A client library for Authentise open services",
        author="Joeseph Rodrigues",
        author_email="dowhilegeek@gmail.com",
        version=get_version(),
        packages=["authentise_services"],
        package_data={"authentise_services": ["authentise_services/*"], },
        include_package_data=True,
        install_requires=[
            "requests==2.7.0",
        ],
        extras_require={
            'develop': [
                'pylint==1.4.3',
                'pytest==2.6.4',
                'httpretty==0.8.10',
                'twine==1.5.0',
            ]},
    )


if __name__ == "__main__":
    main()



"""slicing_settings class module"""
import ntpath  # for windows support

import requests

from authentise_services import errors
from authentise_services.config import Config


class SlicingSettings(object):
    """Class representation of config resources."""
    def __init__(self, session, path=None, engine=None, description=None, url=None):
        self.config = Config()

        try:  # catches session objects and session strings, probably horrible
            self.session = session.session
        except AttributeError:
            self.session = session

        if path:
            if not engine:
                raise errors.ResourceError("resource creation requires path and engine supplied")
            self.upload(path, engine, description)
        elif url:
            self.location = url
            self._get_description()
        else:
            self.description = ""
            self.location = ""

    def upload(self, path, engine, description=None):
        """Create a new config resource in the slicing service and upload the path contents to it"""

        if description is None:
            head, tail = ntpath.split(path)
            description = tail or ntpath.basename(head)

        url = "http://quickslice.{}/config/raw/".format(self.config.host)

        with open(path) as config_file:
            content = config_file.read()

        payload = {"engine": engine,
                   "description": description,
                   "content": content}

        post_resp = requests.post(url, json=payload, cookies={"session": self.session})
        if not post_resp.ok:
            raise errors.ResourceError("upload to slicing service failed")

        self.description = description

        self.location = post_resp.headers["Location"]

    def download(self, path):
        """downloads a config resource to the path"""

        service_get_resp = requests.get(self.location, cookies={"session": self.session})
        payload = service_get_resp.json()

        download_get_resp = requests.get(payload["content"])
        with open(path, "wb") as config_file:
            config_file.write(download_get_resp.content)

    def _get_description(self):
        resp = requests.get(self.location, cookies={"session": self.session})

        self.description = resp.json()["description"]
